# Generated by Django 2.2.4 on 2019-09-01 11:30

from django.db import migrations, models
import lockmgr.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Lock',
            fields=[
                ('name', models.CharField(max_length=100, primary_key=True, serialize=False, verbose_name='Unique Lock Name')),
                ('locked_by', models.CharField(blank=True, max_length=100, null=True)),
                ('lock_process', models.IntegerField(blank=True, default=0, null=True, verbose_name='Process ID of lock requester')),
                ('locked_until', models.DateTimeField(default=lockmgr.models.default_lock_expiry)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Locked At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Last Update')),
            ],
        ),
    ]
